;;;; 2009-10-26 16:51:41
;;;; tests for your lisp code
(in-package :common-lisp-user)

(defpackage #:weblocks-postmodern-unit-tests
  (:use :cl :postmodern :s-sql :weblocks :weblocks-postmodern)
  (:shadowing-import-from :weblocks #:commit-transaction)
  (:shadowing-import-from :s-sql #:text)
  (:documentation
   "Test postmodern store api implementation."))

(in-package :weblocks-postmodern-unit-tests)


;TODO: Write a version with lisp-unit so it works in ecusp.

#|  NOTES:
Remember to create the DB before you run these tests. (test-db.sql) 
The tests are mostly independant but if you run clean-store the db will be empty as advertized.
I use two tables in the db to make sure that the clean-store code works, this is because 
TRUNCATE needs to receive a list of all tables to make sure that foreign references/keys 
dont become an issue. 
|#


(defclass test_1 ()
  ((oid :col-type integer :read-only t
	:accessor oid :ghost t)
   (a_a :col-type string :initarg :a_a
         :accessor a_a)
   (b_b :col-type string :initarg :b_b
                :accessor b_b))
  (:metaclass dao-class)  
  (:keys a_a)
 )

(defclass test_2 ()
  ((oid :col-type integer
	:accessor oid :ghost t)
   (a_a :col-type string :initarg :a_a
         :accessor a_a)
   (b_b :col-type string :initarg :b_b
                :accessor b_b))
  (:metaclass dao-class)
  (:keys a_a))

(setf *oid-p* t)

(defparameter *db-params* 
  (list "weblocks" "postgres" "password" "localhost"))

(defun run-test (name test &rest args) 
  (if  (apply test args)
       (format t "~A => passed~%" name)
       (format t "~A => failed~%" name)))

(defun open-store-test ()
  (let ((db (open-store :postmodern "weblocks" "postgres" "password" "localhost"))
	(passed nil))
    (if db
	(progn
	  (setf passed t)
	  (disconnect db)))
    passed))

(defun close-store-test ()
  (let ((db (connect "weblocks" "postgres" "password" "localhost"))
	(passed nil))
    (if db
	(progn
	  (close-store db)
	  (setf passed t)
	  (disconnect db))
	(format t "Could not aquire db-connection."))
    passed))

(defun clean-store-test ()
  (let ((db (connect "weblocks" "postgres" "password" "localhost"))
	(passed nil))    
    (if db
	(progn
	  (clean-store db)
	  (if (= (with-connection *db-params* 
		     (query (:select (:count '*) :from 'test_1) :single)) 0)
	      (setf passed t))
	  (disconnect db))  
	(format t "Could not aquire db-connection.~%"))
    passed))


(defun dynamic-transaction-test (abort-p)
  (let ((db (apply #'connect *db-params*))
	(passed nil)
	(num (random 5555)))   
    (if db
	(progn
	  (dynamic-transaction 
	   db 
	   #'(lambda ()
	       (execute (format nil  "INSERT INTO test_1 (a_a, b_b)
VALUES ('1aaa_~A', 'bbb_~A');" num num))
	       (format t "Inserted '1aaa_~A.~%" num)
	       (execute (format nil  "INSERT INTO test_1 (a_a, b_b)
VALUES ('2aaa_~A', 'bbb_~A');" num num))
	       (format t "Inserted '1aaa_~A.~%" num)
	       (if abort-p (execute "ABORT"))
	       (execute (format nil  "INSERT INTO test_1 (a_a, b_b)
VALUES ('3aaa_~A', 'bbb_~A');" num num))
	        (format t "Inserted '1aaa_~A.~%" num)))
	  (setf passed t)
	  (disconnect db))  
	(format t "Could not aquire db-connection.~%"))
    passed))


(defun find-persistent-object-by-id-test ()
  (let ((db (apply #'connect *db-params*))
	(passed nil)
	(object nil)
	(oid (with-connection *db-params*
	       (query (:limit (:select 'oid :from 'test_1 ) 1) :single))))
    (if db
	(progn
	  (setf object 
		(find-persistent-object-by-id 
		 db  'test_1 oid))	 
	  (if object
	      (progn  
		(format t "An object with the oid = ~A was found." (oid object))
		(setf passed t)))	 
	  (disconnect db))

	(format t "Could not aquire db-connection.")   
	)
    passed))

(defun persist-object-insert-test ()
  (let ((db (apply #'connect *db-params*))
        (passed nil)
        (object nil))
    (if db
        (progn
          (setf object (make-instance 'test_1 :a_a "Persist!Insert" 
                                      :b_b "Persist!Insert"))          
          (persist-object db object)
          
          (when (slot-boundp object 'oid)
            (progn 
              (format t "An object with the oid = ~A was created." (oid object))
              (if (oid object)	      
                  (setf passed t))))
          (unless (slot-boundp object 'oid)
            (format t "No oid returned with the inserted object. Or object not inserted.")
            (setf passed nil))
          
          (disconnect db))
        (format t "Could not aquire db-connection."))
    passed))

(defun persist-object-update-test ()
  (let ((db (apply #'connect *db-params*))
	(passed nil)
	(object nil))
    (if db
	(progn
	  (setf object (make-instance 'test_1 :a_a "PersistX!Insert" 
				      :b_b "PersistX!Insert"))
	  (persist-object db object)

	  (format t "An object with the oid = ~A was created." (oid object))
	  (setf (a_a object) "Persist!Update")
	  (setf (b_b object) "Persist!Update")

	  (persist-object db object)	  
	  (if (oid object)	      
	      (progn (format t "An object with the oid = ~A was updated." (oid object))
		     (setf passed t)))
	  (disconnect db))
	(format t "Could not aquire db-connection."))
    passed))


(defun delete-persistent-object-test ()
  (let ((db (apply #'connect *db-params*))
	(passed nil)
	(object nil))
    (if db
	(progn
	  (setf object (make-instance 'test_1 :a_a "Todelete!Insert" 
				      :b_b "Todelete!Insert"))
	  (persist-object db object)
	  
	  (format t "An object with the oid = ~A was created." (oid object))
	
	  (delete-persistent-object db object)	
	 
	  (unless (slot-boundp object 'oid)	      
	     (progn (format t "An object was deleted.")
	       (setf passed t)))
	   (disconnect db))
	(format t "Could not aquire db-connection."))
    passed))


(defun find-persistent-objects-test ()
  (let ((db (apply #'connect *db-params*))
        (passed nil)
        (objects nil))
    
    (if db
        (progn
          (setf objects (find-persistent-objects db 'test_1 :select (sql-compile `(:select * :from 'test_1 :where (:> 'oid 0)))
                                                  :order-by '(a_a) :range (cons 0 5))) 
                                                 	 
          (if objects
              (progn  
                (format t "An object with ~A rows was found." (length objects) )
                (setf passed t)))	 
          (disconnect db))
        
        (format t "Could not aquire db-connection.")   
        )
    passed))

(defun count-persistent-objects-test ()
  (let ((db (apply #'connect *db-params*))
        (passed nil)
        (count nil))
    
    (if db
        (progn
          (setf count (count-persistent-objects db 'test_1 :order-by '(a_a) :range (cons 0 5) 
                                                 :test (sql-compile `(:> oid 0))))
                                                 	 
          (if count
              (progn  
                (format t "An object with ~A rows was counted." count )
                (setf passed t)))	 
          (disconnect db))
        
        (format t "Could not aquire db-connection.")   
        )
    passed))

(defun run-all-tests ()
  (run-test 'open-store #'open-store-test)
  (run-test 'close-store #'close-store-test)
  (run-test 'dynamic-transaction #'dynamic-transaction-test nil)
  (run-test 'dynamic-transaction #'dynamic-transaction-test t)
  (run-test 'find-persistent-object-by-id #'find-persistent-object-by-id-test)
  (run-test 'persist-object-insert #'persist-object-insert-test)
  (run-test 'persist-object-update #'persist-object-update-test)
  (run-test 'delete-persistent-object #'delete-persistent-object-test)
  (run-test 'find-persistent-objects #'find-persistent-objects-test)  
  (run-test 'count-persistent-objects #'count-persistent-objects-test)
  
  ;;I told you to run it last...and then to repopulate the db...
  ;(run-test 'clean-store #'clean-store-test)
  )


(defun save-with-oid-test ()
  (let ((db (apply #'connect *db-params*))
        (passed nil)
        (object nil))
    (if db
        (progn
          (setf object (make-instance 'test_1 :a_a "Key 1" 
                                      :b_b "Save!Insert"))          
          (save-dao object)          
          (if (oid object)
              (progn
                (format t "An object with the oid = ~A was created." (oid object))	      
                (setf passed t)))
          ))	   
    (disconnect db)
    (format t "Could not aquire db-connection.")
    passed))

